package com.tzld.longarticle.recommend.server.service.filter.strategy;

import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.model.ContentHisPublishArticle;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.AccountIndexAvgViewCountService;
import com.tzld.longarticle.recommend.server.service.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.filter.FilterStrategy;
import com.tzld.longarticle.recommend.server.service.score.Score;
import com.tzld.longarticle.recommend.server.util.MathUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@Component
@Slf4j
public class LowScoreStrategy implements FilterStrategy {

    @Autowired
    AccountIndexAvgViewCountService accountIndexAvgViewCountService;
    @Autowired
    AccountContentPoolConfigService accountContentPoolConfigService;

    @Override
    public FilterResult filter(FilterParam param) {
        long start = System.currentTimeMillis();
        FilterResult filterResult = new FilterResult();
        if (param.getContents().size() <= 1000) {
            filterResult.setContentIds(param.getContents().stream().map(Content::getId).collect(Collectors.toList()));
            return filterResult;
        }
        filterResult.setContentIds(new ArrayList<>());
        List<Score> scores = new ArrayList<>();
        Map<String, Content> contentMap = param.getContents().stream().collect(Collectors.toMap(Content::getId, c -> c));
        for (Content content : param.getContents()) {
            String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());
            if (!contentPools[2].equals(content.getContentPoolType())) {
                filterResult.getContentIds().add(content.getId());
                continue;
            }
            if (CollectionUtils.isEmpty(content.getHisPublishArticleList())) {
                filterResult.getContentIds().add(content.getId());
                continue;
            }
            double avgViewCountPos = accountIndexAvgViewCountService.getAvgReadCount(param.getGhId(), 3);
            double showViewCountSum = 0D;
            double avgViewCountSum = 0D;
            for (ContentHisPublishArticle hisItem : content.getHisPublishArticleList()) {
                if (hisItem.isInnerAccount() && Objects.nonNull(hisItem.getViewCount())
                        && hisItem.getViewCount() > 0 && Objects.nonNull(hisItem.getAvgViewCount())
                        && hisItem.getAvgViewCount() > 0) {
                    if (!(hisItem.getItemIndex() == 1) && !(hisItem.getItemIndex() == 2)) {
                        showViewCountSum += hisItem.getViewCount();
                        avgViewCountSum += hisItem.getAvgViewCount();
                    }
                }
            }
            double viewCountRate = 0D; // 设置默认值
            double minRate = 5D;
            if (avgViewCountSum > 0) {
                viewCountRate = showViewCountSum / avgViewCountSum;
            }
            double viewCountRateW = MathUtils.sigmoid(avgViewCountSum, 0.0005, avgViewCountPos);
            double viewCountRateScore = 0;
            if (viewCountRate > 0) {
                viewCountRateScore = (Math.min(viewCountRate, minRate) - 1D) * viewCountRateW;
            }
            Score score = new Score();
            score.setContentId(content.getId());
            score.setScore(viewCountRateScore);
            scores.add(score);
        }
        scores.sort((o1, o2) -> -(o1.getScore().compareTo(o2.getScore())));
        filterResult.getContentIds().addAll(scores.subList(0, Math.min(100, scores.size())).stream()
                .map(Score::getContentId).collect(Collectors.toList()));
        if (scores.size() > 100) {
            List<Content> filterContent = scores.subList(100, scores.size() - 1).stream()
                    .map(o -> contentMap.get(o.getContentId())).collect(Collectors.toList());
            filterContent.forEach(o -> o.setFilterReason("低评分内容"));
            filterResult.setFilterContent(filterContent);
        }
        log.info("LowScoreStrategy cost:{}", System.currentTimeMillis() - start);
        return filterResult;
    }
}
